import { earlywarningDetailTableData, earlywarningTableData, investTableData } from "../../data/table/constructData";
import * as defaultConfig from '../../config/defaultConfig'

/**
 * 投资完成情况
 * @param req 
 * @param res 
 */
export async function getInvest(req, res) {
    let line = req.query.line || "全部";
    let year = req.query.year || 2023;
    let tableData = investTableData();
    // let data = change(["line", "year", "年度计划", "年度完成", "年度计划动迁", "年度完成动迁", "累计计划", "累计完成", "累计计划动迁", "累计完成动迁"], tableData["投资完成情况"].bodyList );
    let data = change(["line", "year", "年度累计", "年度投资计划", "年度计划动迁", "年度完成动迁", "总投资额", "建设期累计", "累计计划动迁", "累计完成动迁"], tableData["投资完成情况"].bodyList );

    let ndConf = ["年度累计", "年度投资计划", "年度计划动迁", "年度完成动迁"];
    let ztConf = ["总投资额", "建设期累计", "累计计划动迁", "累计完成动迁"];
    
    let result:any = {};

    let annualAccumulation:any = {};
    let overall:any = {};

    data.forEach(item => {
        let dataLine = item.line;
        let dataYear = item.year;

        if (!annualAccumulation[dataLine]) annualAccumulation[dataLine] = {};
        if (!annualAccumulation["全部"]) annualAccumulation["全部"] = {};

        if (!overall[dataLine]) overall[dataLine] = {};
        if (!overall["全部"]) overall["全部"] = {};

        if (year == dataYear) {
            for (let name = 0; name < ndConf.length; name++) {
                for (let key in item) {
                    if (key.search(ndConf[name]) > -1) {
                        annualAccumulation[dataLine][key] = item[key];

                        if (!annualAccumulation["全部"][key]) annualAccumulation["全部"][key] = 0;
                        annualAccumulation["全部"][key] += item[key];
                    }
                }
            }
    
            for (let name = 0; name < ztConf.length; name++) {
                for (let key in item) {
                    if (key.search(ztConf[name]) > -1) {
                        overall[dataLine][key] = item[key];

                        if (!overall["全部"][key]) overall["全部"][key] = 0;
                        overall["全部"][key] += item[key];
                    }
                }
            }
        }
    
        result["annualAccumulation"] = annualAccumulation;
        result["overall"] = overall;
        
    });

    let defalut = {
        count:0, countRelocation:0, expected:0, expectedRelocation:0, proportion:0
    }
    let annualAccumulationData = annualAccumulation[line] || defalut;
    let overallData = overall[line] || defalut;
    result["annualAccumulation"] = {
        "proportion": annualAccumulationData["年度累计"] == 0 ? 0 : Math.ceil((annualAccumulationData["年度投资计划"] / annualAccumulationData["年度累计"])* 100 )/100 || 0,//年度完成比
        "count": Math.ceil(annualAccumulationData["年度累计"]/10000) || 0,//年度累计投资额
        "countRelocation":Math.ceil( annualAccumulationData["年度完成动迁"]/10000) || 0,//年度累计动迁
        "expected":Math.ceil( annualAccumulationData["年度投资计划"]/10000) || 0,//计划投资额
        "expectedRelocation":Math.ceil( annualAccumulationData["年度计划动迁"]/10000) || 0,//年度计划动迁
    };
    result["overall"] = {
        "proportion": overallData["总投资额"] == 0 ? 0 : Math.ceil((overallData["建设期累计"] / overallData["总投资额"])* 100 )/100 || 0,//年度完成比
        "count": Math.ceil(overallData["建设期累计"]/10000) || 0,//年度累计投资额
        "countRelocation":Math.ceil( overallData["累计完成动迁"]/10000) || 0,//年度累计动迁 单位：(万元)
        "expected":Math.ceil( overallData["总投资额"]/10000) || 0,//计划投资额
        "expectedRelocation":Math.ceil( overallData["累计计划动迁"]/10000) || 0,//年度计划动迁
    };

    res.send(result);

    return result;
}


/**
 * 合同金额预警总览
 * @param req 
 * @param res 
 */
export async function getEarlywarning(req, res) {
    let line = req.query.line || "全部";

    let excelData = earlywarningTableData();

    let result:any = {};

    result["base"] = {
        "accumulatedContractAmountSigned":0,//累计签合同金额
        "estimatedTotalValue":0,//概算总值
        "totalTargetCost":0,//目标成本总值
    };
    let titleList = ["车站", "区间", "轨道", "通讯", "信号", "供电", "综合监控"];
    result["detailedList"] = [];
    if (line == "全部") {
        let map = {};
        for (let key in excelData) {
            let data = change(["总数", "accumulatedContractAmountSigned", "estimatedTotalValue", "totalTargetCost"], excelData[key].bodyList );
            let conf = ["accumulatedContractAmountSigned", "estimatedTotalValue", "totalTargetCost"];



            let dataList = excelData[key].bodyList[excelData[key].bodyList.length -1];
            result["base"].accumulatedContractAmountSigned =  Math.ceil( dataList[1] / 10000 ) || 0;
            result["base"].estimatedTotalValue = Math.ceil( dataList[2] / 10000 ) || 0;
            result["base"].totalTargetCost = Math.ceil( dataList[3] / 10000 ) || 0;

           
            excelData[key].bodyList.forEach(subList => {
                let title = subList[0];
                if (title == "总数") return;
                if (!map[title]) map[title] = {ht:0, gs:0, cb:0};
                
                map[title].ht += subList[1];
                map[title].gs += subList[2];
                map[title].cb += subList[3];
            });
        }
        titleList.forEach(str => {
            let item = map[str];
            result["detailedList"].push({
                type:str,
                estimated: Math.ceil( item.gs / 10000 ),//概算
                targetCost: Math.ceil( item.cb / 10000 ),//目标成本
                accumulatedContractAmountSigned: Math.ceil( item.ht / 10000 ),//累计签合同金额
            });
        });
    } else {
        let checkList = excelData[line]? excelData[line].bodyList : defaultConfig.EarlywarningDefault;
        let allList = checkList[checkList.length-1];
        result["base"].accumulatedContractAmountSigned = Math.ceil( allList[1] / 10000 ) || 0;
        result["base"].estimatedTotalValue = Math.ceil( allList[2] / 10000 ) || 0;
        result["base"].totalTargetCost = Math.ceil( allList[3] / 10000 ) || 0;
        let map = {};
        checkList.forEach(subList => {
            let title = subList[0];
            if (title == "总数") return;
            if (!map[title]) map[title] = {ht:0, gs:0, cb:0};
            
            map[title].ht = subList[1] || 0;
            map[title].gs = subList[2] || 0;
            map[title].cb = subList[3] || 0;
        });
        titleList.forEach(str => {
            let item = map[str];
            result["detailedList"].push({
                type:str,
                estimated: Math.ceil( item.gs / 10000 ),//概算
                targetCost: Math.ceil( item.cb / 10000 ),//目标成本
                accumulatedContractAmountSigned: Math.ceil( item.ht / 10000 ),//累计签合同金额
            });
        });
    }

    res.send(result);
}


/**
 * 合同金额预警总览详细浮窗
 * @param req 
 * @param res 
 */
export async function getDetailedData(req, res) {
    let line = req.query.line || "全部";

    let tableData = earlywarningDetailTableData();
    let result:any = {};

    let detailData = {};
    for (let key in tableData) {
        let gjqjDetailData = change(["line","feeName","accumulatedSignedContractValue", "estimated", "budgetDifference", "targetCost", "targetDifference"], tableData[key].bodyList );
        let dataMap = getDetailData(gjqjDetailData, line, defaultConfig.EarlywarningDetail);

        let detail = [];
        for (let key in dataMap) {
            dataMap[key]['name'] = key;
            detail.push(dataMap[key]);
        }

        detailData[key] = detail;
    }
    
    result["czxxsj"] = detailData;

    res.send(result);
}


function change(titleList, dataList) {
    let data = [];
    dataList.forEach(subList => {
        let onceData:any = {};
        subList.forEach( (item, index)=> {
            let key = titleList[index];
            onceData[key] = item;
        } );
        data.push(onceData);
    });
    return data;
}


function getDetailData(data, line, defaultData) {
    let result:any = {};

    let dataMap = {"全部":{}};

    let strConf = ["accumulatedSignedContractValue", "estimated", "budgetDifference", "targetCost", "targetDifference"];
    data.forEach(subList => {
        let dataLine = subList.line;
        let dataName = subList.feeName;
        
        if (!dataMap[dataLine]) dataMap[dataLine] = {};
        dataMap[dataLine][dataName] = {};
        if (!dataMap["全部"][dataName]) dataMap["全部"][dataName] = {};
        // dataMap[dataLine].push(subList);
        strConf.forEach(str => {
            if (!dataMap[dataLine][dataName][str]) dataMap[dataLine][dataName][str] = 0;
            dataMap[dataLine][dataName][str] += Math.round(subList[str] * 100) / 100;
            
            if (!dataMap["全部"][dataName][str]) dataMap["全部"][dataName][str] = 0;
            dataMap["全部"][dataName][str] += Math.round(subList[str] * 100) / 100;
        })

    })
    
    result = dataMap[line] || defaultData;
    
    return result;
}