/**
 * 建设指挥 计划进度指标面板  逻辑
 * 处理数据
 */

import { relocationTableData, prophaseTableData, scheduleTableData, milestoneTableData } from "../../data/table/constructData";
import * as defaultConfig from '../../config/defaultConfig';

function change(titleList, dataList) {
    let data = [];
    dataList.forEach(subList => {
        let onceData:any = {};
        subList.forEach( (item, index)=> {
            let key = titleList[index];
            onceData[key] = item;
        } );
        data.push(onceData);
    });
    return data;
}


function getProphaseDetailData(data, defaultData, conf, line) {
    let dataMap = {"全部":{"data":{}, "total":0}};
    data.forEach(item => {
        let dataLine = item.line;

        if (!dataMap[dataLine]) dataMap[dataLine] = {data:{}, total:0 };
        dataMap[dataLine]["total"] = item.total;
        dataMap["全部"]["total"] += item.total;
        
        for (let name = 0; name < conf.length; name++) {
            let plan = 0;
            let complete = 0;
            for (let key in item) {
                if (key.search(conf[name]) > -1) {
                    plan = item[conf[name]+"计划数"];
                    complete = item[conf[name]+"完成数"];
                    if (!dataMap[dataLine]["data"][conf[name]]) dataMap[dataLine]["data"][conf[name]] = {plan:0, complete:0};
                    if (!dataMap["全部"]["data"][conf[name]]) dataMap["全部"]["data"][conf[name]] = {plan:0, complete:0};
                }
            }
            dataMap[dataLine]["data"][conf[name]]["plan"] += plan;
            dataMap[dataLine]["data"][conf[name]]["complete"] += complete;

            dataMap["全部"]["data"][conf[name]]["plan"] += plan;
            dataMap["全部"]["data"][conf[name]]["complete"] += complete;
            
        }
        
    });

    let result:any = {};
    result["completionStatus"] = {
        "total":0,
        "complete":0,//完成 
        "incomplete":0,//未完成
    };

    let onceLineData = dataMap[line] || defaultData;
    result["completionStatus"].total = onceLineData.total;
    let complete = 0;
    let scheduleList = [];
    for (let key in onceLineData.data) {
        let subComplete = onceLineData.data[key].complete;
        let subPlan = onceLineData.data[key].plan;
        complete += subComplete;
        scheduleList.push({
            name:key,
            total:subPlan,
            complete:subComplete,
            incomplete:subPlan-subComplete
        });
    }
    result["completionStatus"].complete = complete;
    result["completionStatus"].incomplete =  onceLineData.total - complete;
    result["scheduleList"] = scheduleList;

    return result;
}


/**
 * 前期管理
 * @param req 
 * @param res 
 */
export async function getProphase(req, res) {
    let line = req.query.line || "全部";

    let tableData = prophaseTableData();

    let data = change(["line","year","total","month","单体规划建设方案计划数","单体规划建设方案完成数","单体建设用地规划许可证计划数","单体建设用地规划许可证完成数","单体建设工程规划许可证计划数","单体建设工程规划许可证完成数","单体施工许可证计划数", '单体施工许可证完成数'], tableData["前期管理数据"].bodyList );

    /** dataMap结构{ "线":{ yearCount:0, data:{ "单体规划建设方案":{ plan:0, complete:0 } } } } */
    let conf = ["单体规划建设方案", "单体建设用地规划许可证", "单体建设工程规划许可证", "单体施工许可证"];
    let result:any = getProphaseDetailData(data, defaultConfig.ProphaseDefault, conf, line);
    
    res.send(result);
}


/**
 * 动迁腾地
 * @param req 
 * @param res 
 */
export async function getRelocation(req, res) {
    let line = req.query.line || "总数";
    let year = req.query.year || 2023;

    let tableData = relocationTableData();
    let data = change(["line","total","complete","incomplete"], tableData["概览数据"].bodyList );

    /**基础数据 */
    let dataMap = {};
    data.forEach(info => {
        dataMap[info.line] = {
            "total":info.total,
            "complete":info.complete,
            "incomplete":info.incomplete,
        }
    });

    let result:any = {};

    result["completionStatus"] = dataMap[line] || defaultConfig.CompletionStatusDefault;

    /**详细完成情况 */
    let scheduleTableData = change(["line","year","居民完成率","单位完成率","绿地完成率","居民计划书","居民完成数","单位计划数","单位完成数","绿地计划数","绿地完成数"], tableData["动迁腾地详细"].bodyList );

    let scheduleData = {};
    
    let conf = ["居民完成率", "单位完成率", "绿地完成率"];

    let index = 0;
    scheduleTableData.forEach(item => {
        let dataYear = item.year;
        if (year == dataYear) {
            if (!scheduleData[dataYear]) scheduleData[dataYear] = {data:{} };
            
            for (let name = 0; name < conf.length; name++) {
                for (let key in item){
                    if (!scheduleData[dataYear]["data"][conf[name]]) scheduleData[dataYear]["data"][conf[name]] = {complete:0};

                    if (key.search(conf[name]) > -1) {
                        let complete = item[conf[name]] || 0;
                        scheduleData[dataYear]["data"][conf[name]]["complete"] += complete;
                    }
                }
            }
            
            index ++;
        }
        else {
            scheduleData[year] = defaultConfig.RelocationScheduleData
        }
    });

    let scheduleMap = {};
    for (let key in scheduleData) {
        let {data} = scheduleData[key];
        for (let name in data) {
            let {complete} = data[name];
            if (!scheduleMap[name]) scheduleMap[name] = {complete:0}
            scheduleMap[name].complete += complete;
        }
    }

    let scheduleList = [];
    for (let key in scheduleMap) {
        let subComplete = (scheduleMap[key].complete/index)*100/100 || 0;

        scheduleList.push({
            name:key,
            complete: Math.round(subComplete)
        });
    }

    result["scheduleList"] = scheduleList;

    /**浮窗数据 */
    let detailMap = {}; 

    let detailConf = {
        "居民":[5,6],
        "单位":[7,8],
        "绿地":[9,10]
    };

    tableData["动迁腾地详细"].bodyList.forEach(subList => {
        let dataLine = subList[0];
        if (!detailMap[dataLine]) detailMap[dataLine] = { };

        for (let name in detailConf) {
            let plan = subList[detailConf[name][0]];
            let complete = subList[detailConf[name][1]];
            if (!detailMap[dataLine][name]) detailMap[dataLine][name] = {plan:0, complete:0, incomplete:0};
            detailMap[dataLine][name]["plan"] += plan;
            detailMap[dataLine][name]["complete"] += complete;
            detailMap[dataLine][name]["incomplete"] += plan - complete;
        }

    });

    result["detailList"] = detailMap;

    res.send(result);
}


/**
 * 形象进度
 * @param req 
 * @param res 
 */
export async function getSchedule(req, res) {
    let line = req.query.line || "全部";
    let result:any = {};
    
    let excelData = scheduleTableData();
    
    let data = change(["line","车站总数","车站已完成","车站施工中","车辆基站总数","车辆基站已完成","车辆基站施工中","高架区间总数", '高架区间已完成', '高架区间施工中'], excelData["形象进度数据"].bodyList );
    let conf = ["车站", "车辆基站", "高架区间"]
    let dataMap = {"全部":{"data":{}}};
    
    data.forEach(item => {
        let dataLine = item.line;
        if (!dataMap[dataLine]) dataMap[dataLine] = {data:{} };

        for (let name = 0; name < conf.length; name++) {
            let plan = 0;
            let complete = 0;
            let inProgress = 0;
            for (let key in item){
                if (key.search(conf[name]) > -1) {
                    if (!dataMap[dataLine]["data"][conf[name]]) dataMap[dataLine]["data"][conf[name]] = {plan:0, complete:0, inProgress:0};
                    if (!dataMap["全部"]["data"][conf[name]]) dataMap["全部"]["data"][conf[name]] = {plan:0, complete:0, inProgress:0};
                    plan = item[conf[name]+"总数"];
                    complete = item[conf[name]+"已完成"];
                    inProgress = item[conf[name]+"施工中"];
                }
            }
            
            dataMap[dataLine]["data"][conf[name]]["plan"] += plan;
            dataMap[dataLine]["data"][conf[name]]["complete"] += complete;
            dataMap[dataLine]["data"][conf[name]]["inProgress"] += inProgress;

            dataMap["全部"]["data"][conf[name]]["plan"] += plan;
            dataMap["全部"]["data"][conf[name]]["complete"] += complete;
            dataMap["全部"]["data"][conf[name]]["inProgress"] += inProgress;
        }

    });
    
    let onceLineData = dataMap[line] || defaultConfig.ScheduleDefault;
    let complete = 0;
    let scheduleList = [];
    for (let key in onceLineData.data) {
        let subComplete = onceLineData.data[key].complete;
        let subPlan = onceLineData.data[key].plan;
        let subInProgress = onceLineData.data[key].inProgress;
        complete += subComplete;
        
        scheduleList.push({
            name:key,
            total:subPlan,
            complete:subComplete,
            incomplete:subInProgress,
            notStarted:subPlan-subComplete-subInProgress
        });
    }

    result["scheduleList"] = scheduleList;

    let czDetailData = change(["line","year","计划数量","实际数量"], excelData["车站详细数据"].bodyList );
    result["czDetailedData"] = getDetailData(czDetailData, line, defaultConfig.ScheduleDetailed);
    
    let cljdDetailData = change(["line","year","计划数量","实际数量"], excelData["车辆基地详细数据"].bodyList );
    result["cljdDetailedData"] = getDetailData(cljdDetailData, line, defaultConfig.ScheduleDetailed);
    
    let gjqjDetailData = change(["line","year","计划数量","实际数量"], excelData["高架区间详细数据"].bodyList );
    result["gjqjDetailedData"] = getDetailData(gjqjDetailData, line, defaultConfig.ScheduleDetailed);

    res.send(result);
}


/**
 * 获取形象进度浮窗数据
 * @param data 源数据
 * @param line 线路
 * @param defaultData 默认数据
 * @returns 
 */
function getDetailData(data, line, defaultData) {
    let result:any = {};

    let dataMap = {"全部":{data:{}}};

    data.forEach(subList => {
        let dataLine = subList.line;
        let dataYear = subList.year;
        if (!dataMap[dataLine]) dataMap[dataLine] = {data:{} };

        let plan = subList["计划数量"];
        let complete = subList["实际数量"];

        if (!dataMap[dataLine]["data"][dataYear]) dataMap[dataLine]["data"][dataYear] = {plan:0, complete:0};
        dataMap[dataLine]["data"][dataYear]["plan"] += plan;
        dataMap[dataLine]["data"][dataYear]["complete"] += complete;

        if (!dataMap["全部"]["data"][dataYear]) dataMap["全部"]["data"][dataYear] = {plan:0, complete:0};
        dataMap["全部"]["data"][dataYear]["plan"] += plan;
        dataMap["全部"]["data"][dataYear]["complete"] += complete;
    })

    let onceLineData = dataMap[line] || defaultData;
    let complete = 0;
    let scheduleList = [];
    for (let key in onceLineData.data) {
        let subComplete = onceLineData.data[key].complete;
        let subPlan = onceLineData.data[key].plan;
        complete += subComplete;
        scheduleList.push({
            name:key,
            total:subPlan,
            complete:subComplete
        });
    }
    result = scheduleList;

    return result;
}


export async function getMilestone(req, res) {
    let excelData = milestoneTableData();

    let result:any = {};
    // result["detailedList"] = [{title:"23号线首座车站开工",time:"2022-09-19", desc:"23号线江川东路站于2022-9-19首幅低下连续墙开工。", img:"http://192.168.0.71:40001/2005_01.jpg"},];

    result['detailedList'] = change(["title","time","desc","img"], excelData["里程碑节点完成情况"].bodyList );

    res.send(result);
}


