/**
 * 建设指挥 质量安全指标面板  逻辑
 * 处理数据
 */

import { riskTableData, fatalityTableData } from "../../data/table/constructData";
import { keyValuePackage } from "../../dataPackage/inFrontOut";
import { getOnceYBarChartOut } from "../../dataPackage/out";

/**
 * 致命性风险分布
 * @param req 
 * @param res 
 */
 export function getFatality(req, res) {
    let line = req.query.line || "全部";

    let table = fatalityTableData();

    let 全网 = [];
    let 致命性风险 = [];

    for (let key in table) {
        let {bodyList} = table[key];
        let {count, thisLineCount} = countData(bodyList, line);

        全网.push({key, value:count, unit:""});
        
        let thisCount = line == "全部" ? count : thisLineCount;
        致命性风险.push({key, value:thisCount, unit:""})
    }
    
    let fatalityPackageData = {dataInfo: {"全网": 全网, "致命性风险": 致命性风险}, xUnit: "", yUnit: ""};

    let ret = {};
    ret['fatality'] = getOnceYBarChartOut('致命性风险分布', fatalityPackageData);
 
    res.send(ret);
}


/**
 * 当前风险节点
 * @param req 
 * @param res 
 */
export function getRisk(req, res) {
    let line = req.query.line || "全线网";
    if (line == "全部 ") line == "全线网";

    let table = riskTableData();

    let dataIndex = table["风险详细"].headerList.indexOf(line);

    let 基坑变形 = dataIndex == -1 ? 0 : table["风险详细"].bodyList[0][dataIndex];
    let 地层损失率 = dataIndex == -1 ? 0 : table["风险详细"].bodyList[1][dataIndex];
    let 旁通道地表沉降 = dataIndex == -1 ? 0 :  table["风险详细"].bodyList[2][dataIndex];

    let ret = {};
    ret['jkbx'] = {title: '基坑变形', dataList: {key:"基坑变形", value:基坑变形}};
    ret['dcssl'] = {title: '地层损失率', dataList: {key:"地层损失率", value:地层损失率}};
    ret['ptddbcx'] = {title: '旁通道地表沉降', dataList: {key:"旁通道地表沉降", value:旁通道地表沉降}};

    table = fatalityTableData();

    for (let key in table ) {
        let {allCount,  allRisks,  count, risks} = descData(table[key].bodyList, line);
        let keyStr = key == "盾构区间" ? "区间" : key;
        let onceList = [
            {key:`${keyStr}总数`, value:line == "全线网" ? allCount : count },
            {key:`当前数量`, value:line == "全线网" ? allRisks : risks },
        ];

        let oncePackage = keyValuePackage(onceList);
        let packageData = {title: `${keyStr}详细数据`, dataList: oncePackage};
        switch(key) {
            case "基坑":ret["jkxxsj"] = packageData;break;
            case "盾构区间":ret["qjxxsj"] = packageData;break;
            case "旁通道":ret["ptdxxsj"] = packageData;break;
        }
    }
    res.send(ret);
}


/**
 * 工程安全状态
 * @param req 
 * @param res 
 */
export function getEngineering(req, res) {
    let line = req.query.line || "全线网";
    if (line == "全部 ")line == "全线网";

    let table = fatalityTableData();

    let ret = {};
    for (let key in table ) {
        let {allCount,  allRisks,  count, risks} = descData(table[key].bodyList, line);
        let value = line == "全线网" ? allRisks : risks;
        let title = `${key}风险期数量`;
        
        let packageData = {title, dataList: {key:`风险期数量`, value} };
        switch(key) {
            case "基坑":ret["jkRiskNum"] = packageData;break;
            case "盾构区间":ret["dgqjRiskNum"] = packageData;break;
            case "旁通道":ret["ptdRiskNum"] = packageData;break;
        }

        let {allControl, allFocus, allSecure, control, focus, secure } = getFXQData(table[key].bodyList, line);

        let onceList = [
            {rate:100, key:`安全`, value:line == "全线网" ? allSecure : secure },
            {rate:100, key:`风险可控`, value:line == "全线网" ? allControl : control },
            {rate:100, key:`重点关注`, value:line == "全线网" ? allFocus : focus },
        ];

        let oncePackage = keyValuePackage(onceList);
        let descPackageData = {title: `${key}详细数据`, dataList: oncePackage};
        // descPackageData.dataList.forEach(info => {
        //     info["num"] = info.value;
        // })
        switch(key) {
            case "基坑":ret["jkDetailedData"] = descPackageData;break;
            case "盾构区间":ret["dgqjDetailedData"] = descPackageData;break;
            case "旁通道":ret["ptdDetailedData"] = descPackageData;break;
        }
    }
    res.send(ret);
}


function countData(dataList, name) {
    let count = 0;
    let thisLineCount = 0;
    dataList.forEach(subList => {
        
        let thisCount = 0;
        for (let i = 6; i <= 9; i++) {
            thisCount += subList[i] || 0;
        }

        count += thisCount;
        if (subList[0] == name) thisLineCount = thisCount;
    });

    return {count, thisLineCount};
}


function descData(dataList, name) {
    let allCount = 0;
    let allRisks = 0;
    let count = 0;
    let risks = 0;
    dataList.forEach(subList => {
        allCount += subList[1];
        allRisks += subList[2]; 
        if (subList[0] == name) {
            count += subList[1];
            risks += subList[2]; 
        }
    });
    return { allCount,  allRisks,  count, risks};
}

function getFXQData(dataList, name) {
    let allControl = 0;
    let allFocus = 0;
    let allSecure = 0;
    let control = 0;
    let focus = 0;
    let secure = 0;

    dataList.forEach(subList => {
        allSecure += subList[3];
        allControl += subList[4];
        allFocus += subList[5];
        

        if (subList[0] == name) {
            secure += subList[3];
            control += subList[4];
            focus += subList[5];
        }
    });

    
    return { allControl, allFocus, allSecure, control, focus, secure };
}